@@ -1,70 +1,109 @@
 #### functions ####
 
-# {{{ coxVariableName
+                                        # {{{ coxVariableName
+## * coxVariableName
 #' @title Extract variable names from a model
 #' @description Extract the name of the variables belonging to the linear predictor or used to form the strata
-#' @rdname coxVariableName 
+#' @name coxVariableName
+#' 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package) or \code{cph}
 #'     (rms package).
+#' @param model.frame [data.frame] dataset containing all the relevant variables (entry, time to event, type of event, variables in the linear predictor, strata).
+#' Output from \code{coxModelFrame}.
 #' 
 #' @author Brice Ozenne broz@@sund.ku.dk
-#' 
+
 #' @rdname coxVariableName
 #' @export
-coxVariableName <- function(object){
-  
-  f <- coxFormula(object)
-  special.object <- coxSpecialStrata(object)
-  
-  ## response
-  ls.SurvVar <- SurvResponseVar(f)
-  
-  ## strata
-  xterms <- delete.response(terms(f,
-                                  special = special.object,
-                                  data = coxDesign(object)))
-  ls.StrataInfo <- extractStrata(xterms,
-                                 special = coxSpecialStrata(object))
-  
-  ## regressor
-  lpvars.original <- setdiff(all.vars(f),c(unlist(ls.SurvVar), ls.StrataInfo$strata.vars.original))
-  
-  ## export
-  return(c(list(entry = ls.SurvVar$entry),
-           list(time = ls.SurvVar$time),
-           list(status = ls.SurvVar$status),
-           ls.StrataInfo,
-           list(lpvars = names(coef(object))),
-           list(lpvars.original = lpvars.original)
-  ))
+coxVariableName <- function(object, model.frame){
+
+    ## ** get formula
+    ff <- coxFormula(object)
+
+    ## ** get special operators
+    special.object <- coxSpecial(object)
+
+    ## ** get names of the start/stop/status variables
+    out <- SurvResponseVar(ff)
+
+    ## ** identify special terms
+    xterms <- delete.response(terms(ff,
+                                    special = unlist(special.object),
+                                    data = model.frame))    
+    ls.specials <- attr(xterms,"specials")
+    n.specials <- length(unlist(ls.specials))
+    
+    n.xterms <- length(attr(xterms,"term.labels"))
+    
+    ## ** linear predictor
+    if(n.xterms>n.specials){
+        out$lpvars <- names(coef(object)) ##attr(xterms.lp, "term.labels") not ok for categorical variables with coxph
+        if(n.specials>0){
+            out$lp.sterms <- stats::drop.terms(xterms,unlist(ls.specials))
+        }else{
+            out$lp.sterms <- xterms
+        }
+        out$lpvars.original <- all.vars(out$lp.sterms)
+        
+    }else{
+        out["lpvars"] <- list(NULL)
+        out["lpvars.original"] <- list(NULL)
+        out["lp.sterms"] <- list(NULL)
+    }
+    
+    ## ** strata variables
+    out$strataspecials <- special.object$strata
+    out$is.strata <- length(ls.specials[[special.object$strata]])>0
+
+    if(out$is.strata){
+        if(length(ls.specials[[special.object$strata]])!=n.xterms){
+            out$strata.sterms <- stats::drop.terms(xterms,setdiff(1:n.xterms,ls.specials[[special.object$strata]]))
+        }else{
+            out$strata.sterms <- xterms
+        }
+        out$stratavars <- attr(out$strata.sterms, "term.labels")
+        out$stratavars.original <- all.vars(out$strata.sterms)
+        out$strata.levels <- coxStrataLevel(object)
+
+        if(length(out$strata.levels)==1){
+            out$is.strata <- FALSE
+        }
+    }else{
+        out["stratavars"] <- list(NULL)
+        out["stratavars.original"] <- list(NULL)
+        out["strata.sterms"] <- list(NULL)
+        out$strata.levels <- factor("1") ## not NULL - coherent with coxStrata
+    }
+    
+    ## ** export
+    return(out)
 } 
 # }}}
 
-coxCovars <- function(object){
-  ttobj <- stats::terms(object)
-  ## colnames(attr(ttobj,"factors"))
-  all.vars(attr(delete.response(ttobj),"variables"))
-}
 
 #### methods #####
 
-# {{{ coxBaseEstimator
+                                        # {{{ coxBaseEstimator
+
+
+## * coxBaseEstimator
 #' @title Extract the type of estimator for the baseline hazard
 #' @description Extract the type of estimator for the baseline hazard
-#' @rdname coxBaseEstimator 
+#' @name coxBaseEstimator 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
 #' 
 #' @author Brice Ozenne broz@@sund.ku.dk
-#' 
+
 #' @rdname coxBaseEstimator
 #' @export
 coxBaseEstimator <- function(object){
   UseMethod("coxBaseEstimator") 
 } 
 
+## ** coxBaseEstimator.coxph
 #' @rdname coxBaseEstimator
 #' @method coxBaseEstimator coxph
 #' @export
@@ -72,6 +111,7 @@ coxBaseEstimator.coxph <- function(object){
   return(object$method)
 }
 
+## ** coxBaseEstimator.phreg
 #' @rdname coxBaseEstimator
 #' @method coxBaseEstimator phreg
 #' @export
@@ -80,22 +120,24 @@ coxBaseEstimator.phreg <- function(object){
 }
 # }}}
 
-# {{{ coxCenter
+                                        # {{{ coxCenter
+## * coxCenter
 #' @title Extract the mean value of the covariates
 #' @description Extract the mean value of the covariates
-#' @rdname coxCenter 
+#' @name coxCenter 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
 #' 
 #' @author Brice Ozenne broz@@sund.ku.dk
-#' 
+
 #' @rdname coxCenter
 #' @export
 coxCenter <- function(object){
   UseMethod("coxCenter") 
 } 
 
+## ** coxCenter.cph
 #' @rdname coxCenter
 #' @method coxCenter cph
 #' @export
@@ -103,6 +145,7 @@ coxCenter.cph <- function(object){
   return(setNames(object$means, object$mmcolnames))
 }
 
+## ** coxCenter.coxph
 #' @rdname coxCenter
 #' @method coxCenter coxph
 #' @export
@@ -110,119 +153,161 @@ coxCenter.coxph <- function(object){
   return(setNames(object$means, names(coef(object))))
 }
 
+## ** coxCenter.phreg
 #' @rdname coxCenter
 #' @method coxCenter phreg
 #' @export
 coxCenter.phreg <- function(object){
-  data <- model.matrix(object = eval(object$call$formula),  object$model.frame)[,names(coef(object)), drop = FALSE]
-  return(apply(data,2,mean))
+    return(apply(object$X,2,mean))
 }
-# }}}
+                                        # }}}
 
-# {{{ coxDesign
+                                        # {{{ coxModelFrame
+## * coxModelFrame
 #' @title Extract the design matrix used to train a Cox model
 #' @description Extract the design matrix used to train a Cox model. Should contain the time of event, the type of event, 
 #' the variable for the linear predictor, the strata variables and the date of entry (in case of delayed entry).
-#' @rdname coxDesign 
+#' @name coxModelFrame 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
-#' @param center logical. Should the variable of the linear predictor be centered ?
+#' @param center [logical] Should the variables of the linear predictor be added ?
 #' 
 #' @author Brice Ozenne broz@@sund.ku.dk
-#' 
-#' @rdname coxDesign
+
+#' @rdname coxModelFrame
 #' @export
-coxDesign <- function(object, center){
-  UseMethod("coxDesign") 
+coxModelFrame <- function(object, center){
+  UseMethod("coxModelFrame") 
 } 
 
-#' @rdname coxDesign
-#' @method coxDesign coxph
+## ** coxModelFrame.coxph
+#' @rdname coxModelFrame
+#' @method coxModelFrame coxph
 #' @export
-coxDesign.coxph <- function(object, center = FALSE){
+coxModelFrame.coxph <- function(object, center = FALSE){
+
+    default.start <- 0
   
-  default.start <- 0
+    if("x" %in% names(object) == FALSE){
+        stop("invalid object \n",
+             "set x=TRUE in the call to ",class(object)[1]," \n")
+    }
   
+    if("y" %in% names(object) == FALSE){
+        stop("invalid object \n",
+             "set y=TRUE in the call to ",class(object)[1]," \n")
+    }
+
+    ## ** add x
+    if(NCOL(object[["x"]])!=0){
+        if(center){
+            dt <- as.data.table(rowCenter_cpp(object[["x"]], center = coxCenter(object)))
+        }else{
+            dt <- as.data.table(object[["x"]])
+        }        
+    }else{
+        dt <- NULL
+    }
+    if("strata" %in% names(dt)){
+        stop("The variables in the linear predictor should be named \"strata\" \n")
+    }
+    
+    ## ** add y
+    if(is.null(dt)){
+        dt <- data.table(status = object[["y"]][,"status"])
+    }else{
+        dt[,c("status") := object[["y"]][,"status"]]
+    }
+    if("start" %in% colnames(object$y) == FALSE){
+        dt[,c("start","stop") := list(default.start, object[["y"]][,"time"])]
+    }else{        
+        dt[,c("start","stop") := list(object[["y"]][,"start"], object[["y"]][,"stop"])]
+    }
   
-  if("x" %in% names(object) == FALSE){
-    stop("invalid object \n",
-         "set x=TRUE in the call to ",class(object)[1]," \n")
-  }
-  
-  if("y" %in% names(object) == FALSE){
-    stop("invalid object \n",
-         "set y=TRUE in the call to ",class(object)[1]," \n")
-  }
-  
-  ## set to null to be able to use cbind after 
-  # (otherwise $x may have dimension <0,0> that cannot be bind with $y)
-  if(NCOL(object[["x"]])==0){
-    object[["x"]] <- NULL
-  }else if(center){
-    object[["x"]][] <- rowCenter_cpp(object[["x"]], center = coxCenter(object))
-  }
-  
-  if("strata" %in% names(object) == FALSE){
-    object[["strata"]] <- NULL
-  }
-  if("start" %in% colnames(object$y) == FALSE){
-    object[["y"]] <- cbind(start = default.start, 
-                           stop = object[["y"]][,"time"],
-                           status = object[["y"]][,"status"])
-  }
-  
-  return(as.data.frame(cbind(object[["y"]],
-                             object[["x"]],
-                             strata=object[["strata"]])))
-  
+    ## ** add strata
+    if("strata" %in% names(object)){
+        dt[,c("strata") := object[["strata"]]]
+    }else{
+        dt[,c("strata") := factor(1)]
+    }
+    
+    ## ** export
+    first.col <- c("start","stop","status")
+    data.table::setcolorder(dt, c(first.col,setdiff(names(dt),first.col)))
+    return(dt)
 }
 
-#' @rdname coxDesign
-#' @method coxDesign phreg
+## ** coxModelFrame.cph
+#' @rdname coxModelFrame
+#' @method coxModelFrame cph
 #' @export
-coxDesign.phreg <- function(object, center = FALSE){
-  M.outcome <- as.matrix(object$model.frame[,1])
-  if("entry" %in% names(M.outcome) == FALSE){
-    M.outcome <- cbind(entry = 0, M.outcome)
-  }
+coxModelFrame.cph <- coxModelFrame.coxph
+
+## ** coxModelFrame.phreg
+#' @rdname coxModelFrame
+#' @method coxModelFrame phreg
+#' @export
+coxModelFrame.phreg <- function(object, center = FALSE){
+
+    default.start <- 0
+
+    ## ** add y
+    dt <- as.data.table(unclass(object$model.frame[,1]))
+    if("start" %in% names(dt) == FALSE){
+        dt[, c("start") := default.start]
+    }
   
-  # normalize names
-  name.default <- colnames(M.outcome)
-  name.default<- gsub("entry","start",gsub("time","stop",name.default))
-  colnames(M.outcome) <- name.default
+    ## normalize names
+    name.old <- names(dt)
+    name.new <- gsub("entry","start",gsub("time","stop",name.old))
+    setnames(dt, old = name.old, new = name.new)
   
-  # get covariates
-  M.X <- model.matrix(coxFormula(object), data = object$model.frame)[,names(coef(object)),drop=FALSE]
-  
-  if(center){
-    M.X <- rowCenter_cpp(M.X, center = coxCenter(object))
-  }
-  
-  if("strata" %in% names(object) == FALSE){
-    object[["strata"]] <- NULL
-  }
-  
-  return(as.data.frame(cbind(M.outcome, M.X, strata = object[["strata"]])))
+    ## ** add x
+    if(center){
+        M.X <- rowCenter_cpp(object$X, center = coxCenter(object))
+        colnames(M.X) <- colnames(object$X)
+        dt <- cbind(dt, as.data.table(M.X))
+    }else{
+        dt <- cbind(dt, as.data.table(object$X))
+    }
+    if("strata" %in% names(dt)){
+        stop("The variables in the linear predictor should be named \"strata\" \n")
+    }
+
+    ## ** add strata
+    if(!is.null(object$strata.name)){
+        dt[,c("strata") := as.factor(object$model.frame[[object$strata.name]])]
+    }else{
+        dt[,c("strata") := factor(1)]
+    }
+
+    ## ** export
+    first.col <- c("start","stop","status")
+    data.table::setcolorder(dt, c(first.col,setdiff(names(dt),first.col)))
+    return(dt)
+    
 }
 # }}}
 
-# {{{ coxFormula
+                                        # {{{ coxFormula
+## * coxFormula
 #' @title Extract the formula from a Cox model
 #' @description Extract the formula from a Cox model
-#' @rdname coxFormula 
+#' @name coxFormula 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
 #' 
 #' @author Brice Ozenne broz@@sund.ku.dk
-#' 
+
 #' @rdname coxFormula
 #' @export
 coxFormula <- function(object){
   UseMethod("coxFormula") 
 } 
 
+## ** coxFormula.cph
 #' @rdname coxFormula
 #' @method coxFormula cph
 #' @export
@@ -230,6 +315,7 @@ coxFormula.cph <- function(object){
   return(object$sformula)
 }
 
+## ** coxFormula.coxph
 #' @rdname coxFormula
 #' @method coxFormula coxph
 #' @export
@@ -237,19 +323,20 @@ coxFormula.coxph <- function(object){
   return(object$formula)
 }
 
+## ** coxFormula.phreg
 #' @rdname coxFormula
 #' @method coxFormula phreg
 #' @export
 coxFormula.phreg <- function(object){
-  return(eval(object$call$formula))
+  return(object$formula)
 }
 # }}}
 
 # {{{ coxLP
-
+## * coxLP
 #' @title Compute the linear predictor of a Cox model
 #' @description Compute the linear predictor of a Cox model
-#' @rdname coxLP 
+#' @name coxLP 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
@@ -259,12 +346,14 @@ coxFormula.phreg <- function(object){
 #' @author Brice Ozenne broz@@sund.ku.dk
 #' 
 #' @details In case of empty linear predictor returns a vector of 0 with the same length as the number of rows of the dataset
+
 #' @rdname coxLP
 #' @export
 coxLP <- function(object, data, center){
   UseMethod("coxLP") 
 } 
 
+## ** coxLP.cph
 #' @rdname coxLP
 #' @method coxLP cph
 #' @export
@@ -299,6 +388,7 @@ coxLP.cph <- function(object, data, center){
   return(unname(Xb))
 }
 
+## ** coxLP.coxph
 #' @rdname coxLP
 #' @method coxLP coxph
 #' @export
@@ -316,7 +406,7 @@ coxLP.coxph <- function(object, data, center){
     }
     
   }else{ ## new dataset
-    if(n.varLP>0){
+      if(n.varLP>0){
       is.strata <- attr(object$terms, "special")$strata
       
       
@@ -345,7 +435,6 @@ coxLP.coxph <- function(object, data, center){
       }else{ 
         Xb <- stats::predict(object, newdata = as.data.frame(data), type = "lp")
       }
-      
       if(center == FALSE){
         Xb <- Xb + sum(coxCenter(object)*coef)
       }
@@ -358,31 +447,36 @@ coxLP.coxph <- function(object, data, center){
   return(unname(Xb))
 }
 
+## ** coxLP.phreg
 #' @rdname coxLP
 #' @method coxLP phreg
 #' @export
 coxLP.phreg <- function(object, data, center){
-  coef <- stats::coef(object)
-  n.varLP <- length(coef)
+
+    coef <- stats::coef(object)
+    n.varLP <- length(coef)
+
+    if(n.varLP>0){
+
+        if(is.null(data)){ ## training dataset
+            X <- object$X
+        }else{
+            X <- model.matrix(object, data = data)
+        }
   
-  if(is.null(data)){ ## training dataset
-    data <- object$model.frame
-  }
-  
-  f.term <- delete.response(terms(eval(object$call$formula)))
-  attr(f.term,"intercept") <- 0
-  data <- model.matrix(object = f.term,  data)[,names(coef), drop = FALSE]
-  
-  if(n.varLP>0){
-    Xb <- as.vector(as.matrix(data) %*% coef)
+        Xb <- as.vector(X[,names(coef), drop = FALSE] %*% coef)
     
-    if(center){
-      Xb <- Xb - sum(coxCenter(object)*coef)
-    }
+        if(center){
+            Xb <- Xb - sum(coxCenter(object)*coef)
+        }
     
-  }else{ 
-    Xb <- rep(0, NROW(data)) 
-  } 
+    }else{
+        if(is.null(data)){
+            Xb <- rep(0, coxN(object))
+        }else{
+            Xb <- rep(0, NROW(data))
+        }
+    } 
   
   
   return(Xb)
@@ -390,21 +484,25 @@ coxLP.phreg <- function(object, data, center){
 
 # }}}
 
-# {{{ coxN
+                                        # {{{ coxN
+## * coxN
 #' @title Extract the number of observations from a Cox model
 #' @description Extract the number of observations from a Cox model
-#' @rdname coxN 
+#' @name coxN 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
 #' 
 #' @author Brice Ozenne broz@@sund.ku.dk
+#'
+
 #' @rdname coxN
 #' @export
 coxN <- function(object){
   UseMethod("coxN") 
 } 
 
+## ** coxN.cph
 #' @rdname coxN
 #' @method coxN cph
 #' @export
@@ -412,6 +510,7 @@ coxN.cph <- function(object){
   return(sum(object$n))
 }
 
+## ** coxN.coxph
 #' @rdname coxN
 #' @method coxN coxph
 #' @export
@@ -419,6 +518,7 @@ coxN.coxph <- function(object){
   return(object$n)
 }
 
+## ** coxN.phreg
 #' @rdname coxN
 #' @method coxN phreg
 #' @export
@@ -427,43 +527,104 @@ coxN.phreg <- function(object){
 }
 # }}}
 
-# {{{ coxSpecialStrata
-#' @title Special character for strata in Cox model
-#' @description Return the special character used to indicate the strata variables of the Cox model
-#' @name coxSpecialStrata
+                                        # {{{ coxSpecialStrata
+## * coxSpecial
+#' @title Special characters in Cox model
+#' @description Return the special character(s) of the Cox model, e.g. used to indicate the strata variables.
+#' @name coxSpecial
+#' @param object The fitted Cox regression model object either
+#'     obtained with \code{coxph} (survival package), \code{cph}
+#'     (rms package), or \code{phreg} (mets package).
+#'
+#' @details Must return a list with at least one element strata
+#' indicating the character in the formula marking the variable(s) defining the strata.
+#' 
+#' @author Brice Ozenne broz@@sund.ku.dk
+
+#' @rdname coxSpecial
+#' @export
+coxSpecial <- function(object) UseMethod("coxSpecial")
+
+## ** coxSpecial.coxph
+#' @rdname coxSpecial
+#' @method coxSpecial coxph
+#' @export
+coxSpecial.coxph <- function(object){
+    return(list(strata = "strata",
+                cluster = "cluster"))
+}
+
+## ** coxSpecial.cph
+#' @rdname coxSpecial
+#' @method coxSpecial cph
+#' @export
+coxSpecial.cph <- function(object){
+  return(list(strata = "strat"))
+}
+
+## ** coxSpecial.phreg
+#' @rdname coxSpecial
+#' @method coxSpecial phreg
+#' @export
+coxSpecial.phreg <- function(object){
+    return(list(strata = "strata",
+                cluster = "cluster"))
+}
+# }}}
+
+## * coxStrataLevel
+#' @title Returns the name of the strata in Cox model
+#' @description Return the name of the strata in Cox model
+#' @name coxStrataLevel
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
 #'
 #' @author Brice Ozenne broz@@sund.ku.dk
-#' 
-#' @rdname coxSpecialStrata
-#' @export
-coxSpecialStrata <- function(object) UseMethod("coxSpecialStrata")
 
-#' @rdname coxSpecialStrata
-#' @method coxSpecialStrata coxph
+#' @rdname coxStrataLevel
 #' @export
-coxSpecialStrata.coxph <- function(object){
-  return("strata")
+coxStrataLevel <- function(object) UseMethod("coxStrataLevel")
+
+## ** coxStrataLevel.coxph
+#' @rdname coxStrataLevel
+#' @method coxStrataLevel coxph
+#' @export
+coxStrataLevel.coxph <- function(object){
+    if(!is.null(object$strata)){
+        return(levels(object$strata))
+    }else{
+        return(NULL)
+    }
 }
 
-#' @rdname coxSpecialStrata
-#' @method coxSpecialStrata cph
+## ** coxStrataLevel.cph
+#' @rdname coxStrataLevel
+#' @method coxStrataLevel cph
 #' @export
-coxSpecialStrata.cph <- function(object){
-  return("strat")
+coxStrataLevel.cph <- function(object){
+    if(!is.null(object$strata)){
+        return(levels(object$strata))
+    }else{
+        return(NULL)
+    }
 }
 
-#' @rdname coxSpecialStrata
-#' @method coxSpecialStrata phreg
+## ** coxStrataLevel.phreg
+#' @rdname coxStrataLevel
+#' @method coxStrataLevel phreg
 #' @export
-coxSpecialStrata.phreg <- function(object){
-  return("strata")
+coxStrataLevel.phreg <- function(object){
+    if(!is.null(object$strata.name)){
+       return( levels(object$model.frame[[object$strata.name]]) )
+    }else{
+        return( NULL )
+    }
 }
-# }}}
+                                        # }}}
 
-# {{{ coxStrata
+                                        # {{{ coxStrata
+## * coxStrata
 #' @title Define the strata for a new dataset
 #' @description Define the strata in a dataset to match those of a stratified Cox model
 #' @name coxStrata
@@ -479,15 +640,16 @@ coxSpecialStrata.phreg <- function(object){
 #' @author Brice Ozenne broz@@sund.ku.dk
 #' 
 #' @details if no strata variables returns a vector of \code{"1"} (factor).
-#' 
-#' @rdname coxStrata
-#' @export
-coxStrata <- function(object, data, sterms, strata.vars, levels, strata.levels) UseMethod("coxStrata")
 
 #' @rdname coxStrata
-#' @method coxStrata coxph
 #' @export
-coxStrata.cph <- function(object, data, sterms, strata.vars, levels, strata.levels){
+coxStrata <- function(object, data, sterms, strata.vars, strata.levels) UseMethod("coxStrata")
+
+## ** coxStrata.cph
+#' @rdname coxStrata
+#' @method coxStrata cph
+#' @export
+coxStrata.cph <- function(object, data, sterms, strata.vars, strata.levels){
   
   if(length(strata.vars)==0){ ## no strata variables
     
@@ -495,30 +657,30 @@ coxStrata.cph <- function(object, data, sterms, strata.vars, levels, strata.leve
     strata <- as.factor(rep("1", n))
     
   }else{  ## strata variables
-    
-    if(is.null(data)){ ## training dataset
-      strata <- object$strata
-    }else { ## new dataset
-      tmp <- model.frame(sterms,data)
-      colnames(tmp) <- names(prodlim::parseSpecialNames(names(tmp),"strat"))
-      tmp <- data.frame(lapply(1:NCOL(tmp),function(j){factor(paste0(names(tmp)[j],"=",tmp[,j,drop=TRUE]))}))
-      strata <- apply(tmp,1,paste,collapse=".")
-      if (any(unique(strata) %in% levels == FALSE)){
-        stop("unknown strata: ",paste(unique(strata[strata %in% levels == FALSE]), collapse = " | "),"\n")
+
+      if(is.null(data)){ ## training dataset
+          strata <- object$strata
+      }else { ## new dataset
+          tmp <- model.frame(sterms,data)
+          colnames(tmp) <- names(prodlim::parseSpecialNames(names(tmp),"strat"))
+          tmp <- data.frame(lapply(1:NCOL(tmp),function(j){factor(paste0(names(tmp)[j],"=",tmp[,j,drop=TRUE]))}))
+          strata <- apply(tmp,1,paste,collapse=".")
+          if (any(unique(strata) %in% strata.levels == FALSE)){
+              stop("unknown strata: ",paste(unique(strata[strata %in% strata.levels == FALSE]), collapse = " | "),"\n")
+          }
+          strata <- factor(strata, levels = strata.levels) # add all levels - necessary for predict.CauseSpecificCox to able to correctly convert strata to numeric
       }
-      strata <- factor(strata, levels = levels) # add all levels - necessary for predict.CauseSpecificCox to able to correctly convert strata to numeric
-    }
     
   }
   
   return(strata)
 }
 
-
+## ** coxStrata.coxph
 #' @rdname coxStrata
 #' @method coxStrata coxph
 #' @export
-coxStrata.coxph <- function(object, data, sterms, strata.vars, levels, strata.levels){
+coxStrata.coxph <- function(object, data, sterms, strata.vars, strata.levels){
   
   if(length(strata.vars)==0){ ## no strata variables
     
@@ -531,26 +693,47 @@ coxStrata.coxph <- function(object, data, sterms, strata.vars, levels, strata.le
       strata <- object$strata
     }else { ## new dataset
       strata <- prodlim::model.design(sterms,data=data,xlev=strata.levels,specialsFactor=TRUE)$strata[[1]]
-      if (any(unique(strata) %in% levels == FALSE)){
-        stop("unknown strata: ",paste(unique(strata[strata %in% levels == FALSE]), collapse = " | "),"\n")
+      if (any(unique(strata) %in% strata.levels == FALSE)){
+        stop("unknown strata: ",paste(unique(strata[strata %in% strata.levels == FALSE]), collapse = " | "),"\n")
       }
-      strata <- factor(strata, levels = levels) # add all levels - necessary for predict.CauseSpecificCox to able to correctly convert strata to numeric
+      strata <- factor(strata, levels = strata.levels) # add all levels - necessary for predict.CauseSpecificCox to able to correctly convert strata to numeric
     }
     
   }
   return(strata)
 }
 
+## ** coxStrata.phreg
 #' @rdname coxStrata
 #' @method coxStrata phreg
 # '@export
-coxStrata.phreg <- coxStrata.coxph
+coxStrata.phreg <- function(object, data, sterms, strata.vars, strata.levels){
+  
+  if(length(strata.vars)==0){ ## no strata variables
+    
+    n <- if(is.null(data)) coxN(object) else NROW(data)
+    strata <- as.factor(rep("1", n))
+    
+  }else{  ## strata variables
+      if(is.null(data)){ ## training dataset
+          strata <- object$model.frame[[object$strata.name]]
+      }else { ## new dataset
+          strata <- prodlim::model.design(sterms,data=data,xlev=strata.levels,specialsFactor=TRUE)$strata[[1]]
+          if (any(unique(strata) %in% strata.levels == FALSE)){
+              stop("unknown strata: ",paste(unique(strata[strata %in% strata.levels == FALSE]), collapse = " | "),"\n")
+          }
+      }
+    
+  }
+  return(strata)
+}
 # }}}
 
-# {{{ coxVarCov
+                                        # {{{ coxVarCov
+## * coxVarCov
 #' @title Extract the variance covariance matrix of the beta from a Cox model
 #' @description Extract the variance covariance matrix of the beta from a Cox model
-#' @rdname coxVarCov 
+#' @name coxVarCov 
 #' @param object The fitted Cox regression model object either
 #'     obtained with \code{coxph} (survival package), \code{cph}
 #'     (rms package), or \code{phreg} (mets package).
@@ -559,13 +742,14 @@ coxStrata.phreg <- coxStrata.coxph
 #' 
 #' @details Should return \code{NULL} if the Cox model has no covariate. 
 #' The rows and columns of the variance covariance matrix must be named with the names used in the design matrix.
-#' 
+
 #' @rdname coxVarCov
 #' @export
 coxVarCov <- function(object){
   UseMethod("coxVarCov") 
 } 
 
+## ** coxVarCov.cph
 #' @rdname coxVarCov
 #' @method coxVarCov cph
 #' @export
@@ -580,6 +764,7 @@ coxVarCov.cph <- function(object){
   return(Sigma)
 }
 
+## ** coxVarCov.coxph
 #' @rdname coxVarCov
 #' @method coxVarCov coxph
 #' @export
@@ -594,6 +779,7 @@ coxVarCov.coxph <- function(object){
   return(Sigma)
 }
 
+## ** coxVarCov.phreg
 #' @rdname coxVarCov
 #' @method coxVarCov phreg
 #' @export
@@ -612,10 +798,13 @@ coxVarCov.phreg <- function(object){
 
 #### Auxiliary function #### 
 
-# {{{ SurvResponseVar
+                                        # {{{ SurvResponseVar
+
+## * SurvResponseVar
 #' @title Extract the time and event variable from a Cox model
 #' @description Extract the time and event variable from a Cox model
-#' @rdname SurvResponseVar
+#' @name SurvResponseVar
+#' 
 #' @param formula a formula
 #'
 #' @author Brice Ozenne broz@@sund.ku.dk
@@ -663,61 +852,11 @@ SurvResponseVar <- function(formula){
 }
 # }}}
 
-# {{{ extractStrata
-#' @title Extract the information about the strata
-#' @description Extract the information about the strata stored in a Cox model
-#' @rdname extractStrata
-#' 
-#' @param xterms ...
-#' @param xlevels ...
-#' @param special the special character indicating the strata variables
-#'
-#' @author Brice Ozenne broz@@sund.ku.dk and Thomas A. Gerds tag@@biostat.ku.dk
-#'
-extractStrata <- function(xterms, xlevels = NULL, special){
-  
-  # renamed variables (e.g. strata(X1))
-  xvars <- attr(xterms,"term.labels")
-  strataspecials <- attr(xterms,"specials")[[special]]
-  strata.vars <- xvars[strataspecials]
-  
-  # original variables (e.g. X1)
-  allVars.X <- all.vars(xterms)
-  strata.vars.original <- allVars.X[strataspecials]
-  
-  is.strata <- length(strataspecials)>0
-  
-  if(is.strata){
-    if (length(xvars)>length(strataspecials)){
-      sterms <- stats::drop.terms(xterms,(1:length(xvars))[-strataspecials])
-    } else {
-      sterms <- xterms
-    }
-    
-  }else{
-    sterms <- NULL
-  }
-  
-  # levels of the strata variables [only useful for coxph]
-  if(!is.null(xlevels)){
-    strata.levels <- xlevels[strata.vars]
-  }else{
-    strata.levels <- NULL
-  }
-  
-  return(list(strata.vars = strata.vars,
-              strata.vars.original = strata.vars.original,
-              strataspecials = strataspecials,
-              strata.levels = strata.levels,
-              is.strata = is.strata,
-              sterms = sterms))
-}
-# }}}
-
-# {{{ splitStrataVar
+                                        # {{{ splitStrataVar
+## * splitStrataVar
 #' @title Reconstruct each of the strata variables
 #' @description Reconstruct each of the strata variables from the strata variable stored in the coxph object.
-#' @rdname splitStrataVar
+#' @name splitStrataVar
 #' 
 #' @param object a coxph object.
 #'
@@ -752,10 +891,11 @@ splitStrataVar <- function(object){
 }
 # }}}
 
-# {{{ 
+                                        # {{{
+## * reconstructData
 #' @title Reconstruct the original dataset
 #' @description Reconstruct the original dataset from the elements stored in the coxph object
-#' @rdname reconstructData
+#' @name reconstructData
 #' 
 #' @param object a coxph object.
 #'
@@ -765,10 +905,10 @@ reconstructData <- function(object){
   
   
   ## combine response variable, regressors and strata variable
-  newdata <- as.data.frame(cbind(coxDesign(object),splitStrataVar(object)))
+  newdata <- as.data.frame(cbind(coxModelFrame(object),splitStrataVar(object)))
   
   ## set response variable to their original names
-  infoVar <- coxVariableName(object)
+  infoVar <- coxVariableName(object, model.frame = newdata)
   if(!is.null(infoVar$entry)){
     names(newdata)[names(newdata) == "start"] <- infoVar$entry
   }
@@ -795,7 +935,25 @@ reconstructData <- function(object){
 
 # }}}
 
-# {{{ model.matrix.phreg
+                                        # {{{ model.matrix.phreg
+
+## * model.matrix
+## ** model.matrix.cph
+#' @title Extract design matrix for cph objects
+#' @description Extract design matrix for cph objects
+#' @param object a cph object.
+#' @param data a dataset.
+#' 
+#' @method model.matrix cph
+model.matrix.cph <- function(object, data){
+
+    M <- survival_model.matrix(object, data)[,object$mmcolnames,drop=FALSE]
+    colnames(M) <- colnames(object[["x"]])
+    return(M)
+    
+}
+
+## ** model.matrix.phreg
 #' @title Extract design matrix for phreg objects
 #' @description Extract design matrix for phreg objects
 #' @param object a phreg object.
@@ -805,21 +963,39 @@ reconstructData <- function(object){
 #' 
 #' @method model.matrix phreg
 model.matrix.phreg <- function(object, data){
-  special <- c("strata", "cluster")
-  Terms <- terms(coxFormula(object), special, data = data)
-  
-  ## remove specials
-  if (!is.null(attributes(Terms)$specials$cluster)) {
-    ts <- survival::untangle.specials(Terms, "cluster")
-    Terms <- Terms[-ts$terms]
-  }
-  if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
-    ts <- survival::untangle.specials(Terms, "strata")
-    Terms <- Terms[-ts$terms]
-  }
-  attr(Terms,"intercept") <- 0
-  
-  return(model.matrix(Terms, data))
+    special <- c("strata", "cluster")
+    Terms <- terms(coxFormula(object), special, data = data)
+    
+    ## remove specials
+    if (!is.null(attributes(Terms)$specials$cluster)) {
+        ts <- survival::untangle.specials(Terms, "cluster")
+        Terms <- Terms[-ts$terms]
+    }
+    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
+        ts <- survival::untangle.specials(Terms, "strata")
+        Terms <- Terms[-ts$terms]
+    }
+    attr(Terms,"intercept") <- 1 ## keep intercept to have the same behavior with and without categorical variables 
+
+    missing.var <- setdiff(all.vars(update(object$formula,".~1")), names(data))
+    if(length(missing.var)>0){
+        data[, c(missing.var) := as.list(object$model.frame[1,1][,missing.var])]
+    }
+    X <- model.matrix(Terms, data)
+    return(X[,setdiff(colnames(X),"(Intercept)"),drop=FALSE])
 }
 
 # }}}
+
+## * terms
+## ** terms.phreg
+#' @title Extract terms for phreg objects
+#' @description Extract terms for phreg objects
+#' @param x a phreg object.
+#' @param ... not used.
+#' 
+#' @method terms phreg
+terms.phreg <- function(x, ...){
+    stats::terms(x$formula)
+}
+
