@@ -3,9 +3,9 @@
 ## Author: Thomas Alexander Gerds
 ## Created: Mar 13 2017 (16:53) 
 ## Version: 
-## Last-Updated: Jun 25 2017 (12:31) 
+## Last-Updated: Nov  9 2017 (06:53) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 33
+##     Update #: 35
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -39,7 +39,7 @@
 ##' lr1 = glm(Y~X1+X2+X7+X9,data=learndat,family="binomial")
 ##' lr2 = glm(Y~X3+X5+X6,data=learndat,family="binomial")
 ##' xb=Score(list("LR(X1+X2+X7+X9)"=lr1,"LR(X3+X5+X6)"=lr2),formula=Y~1,
-##'          data=testdat,summary="risks",nullModel=0L)
+##'          data=testdat,summary="risks",null.model=0L)
 ##' plotRisk(xb)
 ##' ## survival
 ##' library(survival)
@@ -48,7 +48,7 @@
 ##' cox1 = coxph(Surv(time,event)~X1+X2+X7+X9,data=learndat,x=TRUE)
 ##' cox2 = coxph(Surv(time,event)~X3+X5+X6,data=learndat,x=TRUE)
 ##' xs=Score(list("Cox(X1+X2+X7+X9)"=cox1,"Cox(X3+X5+X6)"=cox2),formula=Surv(time,event)~1,
-##'          data=testdat,summary="risks",nullModel=0L)
+##'          data=testdat,summary="risks",null.model=0L)
 ##' plotRisk(xs,times=5)
 ##' 
 ##' @export 
@@ -65,8 +65,8 @@ plotRisk <- function(x,
                      cex=1,
                      ...){
     model=ReSpOnSe=risk=status=NULL
-    if (!is.null(x$nullModel))
-        pframe <- x$risks$score[model!=x$nullModel]
+    if (!is.null(x$null.model))
+        pframe <- x$risks$score[model!=x$null.model]
     else
         pframe <- x$risks$score
     if (missing(models)){
@@ -75,7 +75,7 @@ plotRisk <- function(x,
     if (is.na(models[2])) stop("Need two models to scatterplot risks")
     pframe <- pframe[model%in%models]
     modelnames <- pframe[,unique(model)]
-    if (x$responseType=="binary")
+    if (x$response.type=="binary")
         R <- pframe[model==modelnames[1],ReSpOnSe]
     else{
         warning("Under construction")
