@@ -5,14 +5,41 @@ using namespace Rcpp;
 using namespace arma;
 using namespace std;
 
+arma::mat sampleMaxProcess_cpp(int nObject, int nNew, int nSim, const arma::cube& iid, const arma::mat& se);
+
 // {{{ quantileProcess_cpp
 // [[Rcpp::export]]
-
 NumericVector quantileProcess_cpp(int nObject, int nNew, int nSim,
                               arma::cube iid,
                               arma::mat se,
 			      double confLevel){
   
+ 
+  arma::mat maxTime_sample = sampleMaxProcess_cpp(nObject,
+						  nNew,
+						  nSim,
+						  iid,
+						  se);
+  
+  int indexQuantile = round(nSim * confLevel);
+  colvec tempo;
+  NumericVector Vquantile(nNew);
+    
+  for (int iCol = 0; iCol < nNew; iCol++){
+    tempo = sort(maxTime_sample.col(iCol));
+    Vquantile[iCol] = tempo[indexQuantile];
+  }
+  
+  return(Vquantile);
+}
+// }}}
+
+// {{{ sampleMaxProcess_cpp
+// [[Rcpp::export]]
+arma::mat sampleMaxProcess_cpp(int nObject, int nNew, int nSim,
+                              const arma::cube& iid,
+                              const arma::mat& se){
+
   void GetRNGstate(),PutRNGstate(); 
   GetRNGstate();
 
@@ -29,20 +56,10 @@ NumericVector quantileProcess_cpp(int nObject, int nNew, int nSim,
     maxTime_sample(iSim, iCol) = iidG.col(iCol).max();
   }
   }
-
-  int indexQuantile = round(nSim * confLevel);
-  colvec tempo;
-  NumericVector Vquantile(nNew);
-    
-  for (int iCol = 0; iCol < nNew; iCol++){
-    tempo = sort(maxTime_sample.col(iCol));
-    Vquantile[iCol] = tempo[indexQuantile];
-  }
-  
- // arma::mat res2 = iid.each_slice() * theta;
   
   PutRNGstate();
   
   
-  return(Vquantile);
+  return(maxTime_sample);
 }
+// }}}
